!****************************************************************************************
! One tax
! Optimal taxation
! by Fernando Martin (2003)
!
! Governments sets income tax for today
! Complete Utility Function
! G is endogenous
! LOCAL APPROXIMATION: POLYNOMIAL SOLUTION
! Solution is found using simultaneous
!****************************************************************************************


Module environment


Implicit none

! Precision parameters

integer,    parameter :: prec     = 8		! double precision
real(prec), parameter :: epsilon  = 1d-13	! auxiliar for IFs
real(prec), parameter :: tol      = 1d-07	! precision of solution
real(prec), parameter :: D        = 1d-03	! maximum increment in all derivatives
real(prec), parameter :: dstep    = 1d-02	! initial step of DDERIV (use 1d-03 if ls=2)
real(prec), parameter :: dtol     = 1d-03	! tolerance of DDERIV
integer,    parameter :: ls       = 1		! type of tax (1=income,2=capital,3=labor)
integer,    parameter :: deduct   = 0		! if 1 then depreciation is not tax deductible


! Global parameters

real(prec), parameter :: beta  = 0.96_prec		! discount factor
real(prec), parameter :: gamma = 0.36_prec		! capital share
real(prec), parameter :: delta = 0.08_prec		! deprecitation rate
real(prec), parameter :: A     = 1.32477_prec 	! TFP parameter

real(prec) :: psi   = 0.000_prec	! elasticity of private-public goods substitution
real(prec) :: rho   = 0.000_prec	! elasticity of consumption-leisure substitution
real(prec) :: sigma = 1.000_prec	! elasticity of intertemporal subsitution

real(prec) :: alpha(3) = (/0.30_prec,0.70_prec,0.13_prec/)	! preference parameters over C, L and G


! Polynomial variables

integer, parameter :: q = 4		! number of coefficients
real(prec) :: ck(q), cl(q), ct(q)	! coefficients
real(prec) :: sol(q,1+q*3), guess(1+q*3)


! NLES variables

integer :: i
real(prec) :: solver, fnorm, DDERIV

external :: solver, DNEQNF, DDERIV


Contains


!***********************************************************************************
!	FUNCTIONS
!***********************************************************************************


! 5-point numerical derivative

real(prec) function numder5(Func,x,h,n)
	Implicit none
	real(prec) :: Func, F, F1, F2, F3, F4, x, x1, x2, x3, x4, h
	integer :: n
	external :: Func
	
	x1 = x - 2d-0 * h
	x2 = x - h
	x3 = x + h
	x4 = x + 2d-0 * h

	F  = Func(x)
	F1 = Func(x1)
	F2 = Func(x2)
	F3 = Func(x3)
	F4 = Func(x4)
	
	if (n==0) numder5 = F
	if (n==1) numder5 = (F1 - 8d-0*F2 + 8d-0*F3 - F4) / (12d-0*h)
	if (n==2) numder5 = (-F1 + 16d-0*F2 -30d-0*F + 16d-0*F3 - F4) / (12d-0*h**2d-0)
	if (n==3) numder5 = (-F1 + 2d-0*F2 - 2d-0*F3 + F4) / (2d-0*h**3d-0)
	if (n==4) numder5 = (F1 - 4d-0*F2 + 6d-0*F - 4d-0*F3 + F4) / (h**4d-0)
	
end function numder5


! Polynomial Function

real(prec) function poly(x,c,arg)
	Implicit none
	real(prec) :: x, c(:), p(2)
	integer :: arg, j, n

	n = size(c)

	p(1) = c(n)
	p(2) = 0d-0
	
	do j = n-1, 1, -1
		p(2) = p(2)*x + p(1)
		p(1) = p(1)*x + c(j)
	end do

	if (arg==0) poly = p(1)
	if (arg==1) poly = p(2)

end function poly


! Production Function

real(prec) function prodfun(k,l)
	Implicit none
	real(prec) :: k, l
	prodfun = A * k**gamma * l**(1-gamma)
end function prodfun


! Wage

real(prec) function wage(k,l)
	Implicit none
	real(prec) :: k, l
	if (k>=0 .and. l>=0) wage = A * (1-gamma) *   k **gamma *   l **(-gamma)
	if (k< 0 .and. l>=0) wage = A * (1-gamma) *-(-k)**gamma *   l **(-gamma)
	if (k>=0 .and. l< 0) wage = A * (1-gamma) *   k **gamma *-(-l)**(-gamma)
	if (k< 0 .and. l< 0) wage = A * (1-gamma) *-(-k)**gamma *-(-l)**(-gamma)
end function wage


! Rate of Return

real(prec) function rate(k,l)
	Implicit none
	real(prec) :: k, l
	if (k>=0 .and. l>=0) rate = A * gamma *   k **(gamma-1) *   l **(1-gamma)
	if (k< 0 .and. l>=0) rate = A * gamma *-(-k)**(gamma-1) *   l **(1-gamma)
	if (k>=0 .and. l< 0) rate = A * gamma *   k **(gamma-1) *-(-l)**(1-gamma)
	if (k< 0 .and. l< 0) rate = A * gamma *-(-k)**(gamma-1) *-(-l)**(1-gamma)
end function rate


! Government Function

real(prec) function govfun(k,l,t)
	Implicit none
	real(prec) :: k, l, t
	if (ls==1) govfun = t * ( (rate(k,l)-delta)*k + wage(k,l)*l )
	if (ls==2) govfun = t *   (rate(k,l)-delta)*k
	if (ls==3) govfun = t * wage(k,l)*l

	if (ls==1 .and. deduct==1) govfun = t * prodfun(k,l)
	if (ls==2 .and. deduct==1) govfun = t * rate(k,l)*k
end function govfun


! Consumption Function

real(prec) function consfun(k,kk,l,t)
	Implicit none
	real(prec) :: k, kk, l, t
	consfun = (rate(k,l)-delta)*k + wage(k,l)*l + k - kk - govfun(k,l,t)
end function consfun


! Marginal Utility for the complete utility function

real(prec) function dU(C,L,G,arg)
	Implicit none
	real(prec) :: C, L, G, XX, YY
	integer    :: arg
	
	if (sigma==1) then
		
		if(arg==1) dU = Uc(C)
		if(arg==2) dU = Ul(L)
		if(arg==3) dU = Ug(G)

	else
	
		YY = alpha(1)*C**rho + alpha(2)*L**rho
		XX = (1-alpha(3)) * YY**(psi/rho) + alpha(3) * G**psi

		if(arg==1) dU = alpha(1)*(1-alpha(3)) * XX**((1-sigma-psi)/psi) * YY**((psi-rho)/rho) * C**(rho-1)
		if(arg==2) dU = alpha(2)*(1-alpha(3)) * XX**((1-sigma-psi)/psi) * YY**((psi-rho)/rho) * L**(rho-1)
		if(arg==3) dU = alpha(3) * XX**((1-sigma-psi)/psi) * G**(psi-1)
	
	end if

end function dU


! Marginal Utility of Consumption

real(prec) function Uc(C)
	Implicit none
	real(prec) :: C
	Uc = alpha(1)*(1-alpha(3)) / C
	if (C<0) Uc=-9999.9_prec
end function Uc


! Marginal Utility of Leisure

real(prec) function Ul(L)
	Implicit none
	real(prec) :: L
	Ul = alpha(2)*(1-alpha(3)) / L
	if (L<0) Ul=-9999.9_prec
end function Ul


! Marginal Utility of Public Good

real(prec) function Ug(G)
	Implicit none
	real(prec) :: G
	Ug = alpha(3) / G
	if (G<0) Ug=-9999.9_prec
end function Ug


!***** Euler Equations and Derivatives *********************************************

! Dynamic Euler Equation

real(prec) function euler(k,kk,kkk,l,ll,t,tt)
	Implicit none
	real(prec) :: k, kk, kkk, l, ll, t, tt, U_c, UU_c
	U_c = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
	UU_c= dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),1)

	if (ls==1) euler = -U_c + beta * UU_c * ((1-tt)*(rate(kk,ll)-delta) + 1)
	if (ls==2) euler = -U_c + beta * UU_c * ((1-tt)*(rate(kk,ll)-delta) + 1)
	if (ls==3) euler = -U_c + beta * UU_c * (rate(kk,ll) - delta + 1)

	if (ls==1 .and. deduct==1) euler = -U_c + beta * UU_c *  (1-tt)*rate(kk,ll)
	if (ls==2 .and. deduct==1) euler = -U_c + beta * UU_c * ((1-tt)*rate(kk,ll) - delta + 1)

end function euler


! Static Consumption-Leisure Equation

real(prec) function stateq(k,kk,l,t)
	Implicit none
	real(prec) :: k, kk, l, t, U_c, U_l
	U_c = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
	U_l = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),2)

	if (ls==1) stateq = U_c * (1-t)*wage(k,l) - U_l
	if (ls==2) stateq = U_c *       wage(k,l) - U_l
	if (ls==3) stateq = U_c * (1-t)*wage(k,l) - U_l

end function stateq


! Derivatives of the Euler Equation **********************************

real(prec) function dE(k,kk,kkk,l,ll,t,tt,arg)
	Implicit none
	real(prec) :: k,kk,kkk,l,ll,t,tt
	integer :: arg
	if (arg==1) dE = (euler(k+D,kk,kkk,l,ll,t,tt) - euler(k-D,kk,kkk,l,ll,t,tt)) / (2d-0*D)
	if (arg==2) dE = (euler(k,kk+D,kkk,l,ll,t,tt) - euler(k,kk-D,kkk,l,ll,t,tt)) / (2d-0*D)
	if (arg==3) dE = (euler(k,kk,kkk+D,l,ll,t,tt) - euler(k,kk,kkk-D,l,ll,t,tt)) / (2d-0*D)
	if (arg==4) dE = (euler(k,kk,kkk,l+D,ll,t,tt) - euler(k,kk,kkk,l-D,ll,t,tt)) / (2d-0*D)
	if (arg==5) dE = (euler(k,kk,kkk,l,ll+D,t,tt) - euler(k,kk,kkk,l,ll-D,t,tt)) / (2d-0*D)
	if (arg==6) dE = (euler(k,kk,kkk,l,ll,t+D,tt) - euler(k,kk,kkk,l,ll,t-D,tt)) / (2d-0*D)
	if (arg==7) dE = (euler(k,kk,kkk,l,ll,t,tt+D) - euler(k,kk,kkk,l,ll,t,tt-D)) / (2d-0*D)
end function dE


! Derivatives of the Static Equation *********************************

real(prec) function dS(k,kk,l,t,arg)
	Implicit none
	real(prec) :: k,kk,l,t
	integer :: arg
	if (arg==1) dS = (stateq(k+D,kk,l,t) - stateq(k-D,kk,l,t)) / (2d-0*D)
	if (arg==2) dS = (stateq(k,kk+D,l,t) - stateq(k,kk-D,l,t)) / (2d-0*D)
	if (arg==3) dS = (stateq(k,kk,l+D,t) - stateq(k,kk,l-D,t)) / (2d-0*D)
	if (arg==4) dS = (stateq(k,kk,l,t+D) - stateq(k,kk,l,t-D)) / (2d-0*D)
end function dS


!***** Generalized Euler Equations *************************************************

real(prec) function gee(k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk)

	Implicit none

    ! Define variables

    real(prec) :: k, kk, kkk, kkkk
    real(prec) :: l, ll, lll
    real(prec) :: t, tt, ttt
    real(prec) :: hhk, llk, ttk, ppk
    real(prec) :: U_c, UU_c, U_l, UU_l, U_g, UU_g
    real(prec) :: C_l, C_t, CC_k, CC_l, CC_t
	real(prec) :: G_l, G_t, GG_k, GG_l, GG_t
    real(prec) :: E, X, XX, Y, YY, ZZ
	real(prec) :: a0, a1, a2, a3, a4, a5
    real(prec) :: Eta_kk, Eta_l, Eta_t, Etap_k, Etap_l, Etap_t
    real(prec) :: Eps_kk, Eps_l, Eps_t, Epsp_k, Epsp_l, Epsp_t
    real(prec) :: lamda, lamdap, mu, mup
	

	! Get Utility Function Derivatives

    U_c   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
    U_l   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),2)
	U_g   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),3)

	UU_c  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),1)
    UU_l  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),2)
    UU_g  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),3)


    ! Get Euler Equation Derivatives

    Eta_kk = dE(k,kk,kkk,l,ll,t,tt,2)       +  dE(k,kk,kkk,l,ll,t,tt,3) * hhk + &
         &   dE(k,kk,kkk,l,ll,t,tt,5) * llk +  dE(k,kk,kkk,l,ll,t,tt,7) * ttk
    Eta_l =  dE(k,kk,kkk,l,ll,t,tt,4)
	Eta_t =  dE(k,kk,kkk,l,ll,t,tt,6)
    
    Etap_k = dE(kk,kkk,kkkk,ll,lll,tt,ttt,1)
    Etap_l = dE(kk,kkk,kkkk,ll,lll,tt,ttt,4)
	Etap_t = dE(kk,kkk,kkkk,ll,lll,tt,ttt,6)
    

    ! Get Static Equation Derivatives

    Eps_kk = dS(k,kk,l,t,2)
    Eps_l  = dS(k,kk,l,t,3)
    Eps_t  = dS(k,kk,l,t,4)

    Epsp_k = dS(kk,kkk,ll,tt,1)
    Epsp_l = dS(kk,kkk,ll,tt,3)
    Epsp_t = dS(kk,kkk,ll,tt,4)


    ! Get additional derivatives

	C_l = (consfun(k,kk,l+D,t) - consfun(k,kk,l-D,t)) / (2d-0*D)
	C_t = (consfun(k,kk,l,t+D) - consfun(k,kk,l,t-D)) / (2d-0*D)
	
	CC_k = (consfun(kk+D,kkk,ll,tt) - consfun(kk-D,kkk,ll,tt)) / (2d-0*D)
	CC_l = (consfun(kk,kkk,ll+D,tt) - consfun(kk,kkk,ll-D,tt)) / (2d-0*D)
	CC_t = (consfun(kk,kkk,ll,tt+D) - consfun(kk,kkk,ll,tt-D)) / (2d-0*D)

	G_l = (govfun(k,l+D,t) - govfun(k,l-D,t)) / (2d-0*D)
	G_t = (govfun(k,l,t+D) - govfun(k,l,t-D)) / (2d-0*D)
	
	GG_k = (govfun(kk+D,ll,tt) - govfun(kk-D,ll,tt)) / (2d-0*D)
	GG_l = (govfun(kk,ll+D,tt) - govfun(kk,ll-D,tt)) / (2d-0*D)
	GG_t = (govfun(kk,ll,tt+D) - govfun(kk,ll,tt-D)) / (2d-0*D)


	! Exact solutions

	if (ls==1 .and. deduct==0) then

		C_l  = (1-t) * wage(k,l)
		CC_l = (1-tt) * wage(kk,ll)
		C_t  = -((rate(k,l)-delta) * k + wage(k,l) * l )
		CC_t = -((rate(kk,ll)-delta) * kk + wage(kk,ll) * ll)
		CC_k = (1-tt) * (rate(kk,ll)-delta) + 1

		G_l  = t * wage(k,l)
		GG_l = tt * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (rate(kk,ll)-delta)

	end if

	if (ls==2 .and. deduct==0) then

		C_l  = (1-t*gamma) * wage(k,l)
		CC_l = (1-tt*gamma) * wage(kk,ll)
		C_t  = -(rate(k,l)-delta) * k
		CC_t = -(rate(kk,ll)-delta) * kk
		CC_k = (1-tt) * (gamma*rate(kk,ll)-delta) + (1-gamma)*rate(kk,ll) + 1

		G_l  = t * gamma * wage(k,l)
		GG_l = tt * gamma * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (gamma*rate(kk,ll)-delta)

	end if

	if (ls==3) then

		C_l  = (1-t*(1-gamma)) * wage(k,l)
		CC_l = (1-tt*(1-gamma)) * wage(kk,ll)
		C_t  = -wage(k,l) * l
		CC_t = -wage(kk,ll) * ll
		CC_k = (1-tt*(1-gamma)) * rate(kk,ll) + 1 - delta

		G_l  = t * (1-gamma) * wage(k,l)
		GG_l = tt * (1-gamma) * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (1-gamma) * rate(kk,ll)

	end if

	
	! Set coefficients of GEE

    a0 = (Eps_kk*Eta_l - Eps_l*Eta_kk) / (Eps_l*Eta_t - Eps_t*Eta_l)
    a1 = (Eps_t*Eta_kk - Eps_kk*Eta_t) / (Eps_l*Eta_t - Eps_t*Eta_l)
    a2 = beta
    a3 = beta * (Epsp_k*Etap_l - Epsp_l*Etap_k) / (Epsp_l*Etap_t - Epsp_t*Etap_l)
    a4 = beta * (Epsp_t*Etap_k - Epsp_k*Etap_t) / (Epsp_l*Etap_t - Epsp_t*Etap_l)


    ! Define expressions

	E = -U_c + beta * UU_c * CC_k
    X  = U_c  * C_t  + U_g  * G_t
    XX = UU_c * CC_t + UU_g * GG_t
    Y  = U_c  * C_l  - U_l  + U_g  * G_l
    YY = UU_c * CC_l - UU_l + UU_g * GG_l
    ZZ = UU_g * GG_k
	

	! Get GEE
	
	GEE = E + a0 * X + a1 * Y + a2 * ZZ + a3 * XX + a4 * YY

end function gee


! Derivatives of GEE *********************************

real(prec) function dGEE(k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk,arg)
	Implicit none
	real(prec) :: k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,p,pp,ppp,hhk,llk,ttk
	integer :: arg
	if (arg== 1) dGEE = (GEE(k+D,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k-D,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 2) dGEE = (GEE(k,kk+D,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk-D,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 3) dGEE = (GEE(k,kk,kkk+D,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk-D,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 4) dGEE = (GEE(k,kk,kkk,kkkk+D,l,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk-D,l,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 5) dGEE = (GEE(k,kk,kkk,kkkk,l+D,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l-D,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 6) dGEE = (GEE(k,kk,kkk,kkkk,l,ll+D,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll-D,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 7) dGEE = (GEE(k,kk,kkk,kkkk,l,ll,lll+D,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll,lll-D,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 8) dGEE = (GEE(k,kk,kkk,kkkk,l,ll,lll,t+D,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll,lll,t-D,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 9) dGEE = (GEE(k,kk,kkk,kkkk,l,ll,lll,t,tt+D,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll,lll,t,tt-D,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg==10) dGEE = (GEE(k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt+D,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt-D,hhk,llk,ttk)) / (2d-0*D)
end function dGEE


End Module environment



!*********************************************************************************************


!******************************************************************************************
!	EXTERNAL FUNCTIONS
!******************************************************************************************


real(prec) function FEE(k)
	
	Use environment
	Implicit none

	real(prec) :: k, h, hh, l, ll, t, tt

	h  = poly(k,ck,0)
	l  = poly(k,cl,0)
	t  = poly(k,ct,0)
	hh = poly(h,ck,0)
	ll = poly(h,cl,0)
	tt = poly(h,ct,0)

	FEE = euler(k,h,hh,l,ll,t,tt)

end function FEE


real(prec) function FSE(k)
	
	Use environment
	Implicit none

	real(prec) :: k, h, l, t

	h  = poly(k,ck,0)
	l  = poly(k,cl,0)
	t  = poly(k,ct,0)
	
	FSE = stateq(k,h,l,t)

end function FSE


real(prec) function FGEE(k)
	
	Use environment
	Implicit none

	real(prec) :: k, h, hh, hhh, l, ll, lll, t, tt, ttt, hhk, llk, ttk

	h   = poly(k ,ck,0)
	l   = poly(k ,cl,0)
	t   = poly(k ,ct,0)
	hh  = poly(h ,ck,0)
	ll  = poly(h ,cl,0)
	tt  = poly(h ,ct,0)
	hhh = poly(hh,ck,0)
	lll = poly(hh,cl,0)
	ttt = poly(hh,ct,0)
	hhk = poly(h ,ck,1)
	llk = poly(h ,cl,1)
	ttk = poly(h ,ct,1)

	FGEE = gee(k,h,hh,hhh,l,ll,lll,t,tt,ttt,hhk,llk,ttk)

end function FGEE



!******************************************************************************************
!	SOLVER SUBROUTINE
!******************************************************************************************


subroutine solver(X,F,n)

	use environment
	Implicit none

	integer :: j, m, n
	real(prec) :: x(n), F(n), k, s, t
	real(prec) :: FEE, FSE, FGEE
	external   :: FEE, FSE, FGEE

	m  = (n-1) / 3		! set number of coefficients

	k  = x(1)

	do j = 1, m
		ck(j) = x(j+2*(j-1)+1)
		cl(j) = x(j+2*(j-1)+2)
		ct(j) = x(j+2*(j-1)+3)
	end do


	! System of Equations

	if (k>0d-0 .and. poly(k,cl,0)<1d-0 .and. consfun(k,poly(k,ck,0),poly(k,cl,0),poly(k,ct,0))>0d-0) then

		F(1) = poly(k,ck,0) - k
		F(2) = FEE(k)
		F(3) = FSE(k)
		F(4) = FGEE(k)

		do j = 2, m, 1

			s = dstep**(1d-0/(j-1))	! initial step
			t = dtol **(1d-0/(j-1))	! tolerance

			F(j+2*(j-1)+1) = DDERIV(FEE ,j-1,k,s,t)
			F(j+2*(j-1)+2) = DDERIV(FSE ,j-1,k,s,t)
			F(j+2*(j-1)+3) = DDERIV(FGEE,j-1,k,s,t)

		end do

	else

		F = 10d-0

	end if

	if(m==1) write(5,"(4F15.10)") F
	if(m==2) write(5,"(7F15.10)") F
	if(m==3) write(5,"(10F15.10)") F
	if(m==4) write(5,"(13F15.10)") F

end subroutine solver



!******************************************************************************************
!	DISPLAY SUBROUTINE
!******************************************************************************************


subroutine display(x)

	use environment
	Implicit none

	real(prec) :: x(1+q*3), F(6)
	real(prec) :: k,l,t,hk,lk,tk
	integer :: j
	real(prec) :: FEE,FSE,FGEE
	external   :: FEE,FSE,FGEE

	k  = x(1)

	do j = 1, q
		ck(j) = x(j+2*(j-1)+1)
		cl(j) = x(j+2*(j-1)+2)
		ct(j) = x(j+2*(j-1)+3)
	end do

	l  = poly(k,cl,0)
	t  = poly(k,ct,0)
	
	hk = poly(k,ck,1)
	lk = poly(k,cl,1)
	tk = poly(k,ct,1)


	! System of Equations

	F(1) = euler(k,k,k,l,l,t,t)
	
	F(2) = stateq(k,k,l,t)
	
	F(3) = gee(k,k,k,k,l,l,l,t,t,t,hk,lk,tk)

	F(4) =  dE(k,k,k,l,l,t,t,1) + dE(k,k,k,l,l,t,t,2)*hk + dE(k,k,k,l,l,t,t,3)*hk*hk &
		        			    + dE(k,k,k,l,l,t,t,4)*lk + dE(k,k,k,l,l,t,t,5)*lk*hk &
							    + dE(k,k,k,l,l,t,t,6)*tk + dE(k,k,k,l,l,t,t,7)*tk*hk

	F(5) =  dS(k,k,l,t,1) + dS(k,k,l,t,2)*hk + dS(k,k,l,t,3)*lk + dS(k,k,l,t,4)*tk

	F(4) = numder5(FEE ,k,5d-5,1)
	F(5) = numder5(FSE ,k,5d-5,1)
	F(6) = numder5(FGEE,k,5d-5,1)
	
	write(5,*)
	write(5,*) k
	write(5,*) l
	write(5,*) t
	write(5,*)
	write(5,*) hk
	write(5,*) lk
	write(5,*) tk
	write(5,*)
	write(5,"(F20.16)") F
	write(5,*)
	write(5,"(F20.16)") x
	write(5,*)


end subroutine display



!******************************************************************************************
!	PROGRAM
!******************************************************************************************


Program tax

use environment
use imslf90

call ERSET (0, 1, 0)	! tells ISLM not to stop if there's a fatal error

open(unit=5,file='tax.out')			! solutions
open(unit=3,file='solution.out')	! interpolated solutions


! Make guess

if (ls==1) guess(1:7) = (/1.836_prec,0.275_prec,0.344_prec,0.175_prec,0.850_prec,-0.047_prec,0.041_prec/)
if (ls==2) guess(1:7) = (/0.551_prec,0.111_prec,0.323_prec,0.478_prec,0.799_prec,-0.067_prec,0.623_prec/)
if (ls==3) guess(1:7) = (/2.127_prec,0.332_prec,0.339_prec,0.258_prec,0.844_prec,-0.041_prec,0.019_prec/)

!guess(1:7) = (/0.607_prec,0.112_prec,0.322_prec,0.516_prec,0.816_prec,-0.063_prec,0.114_prec/)
!guess(1:7) = (/0.50_prec,0.112_prec,0.322_prec,0.026_prec,0.816_prec, 0.000_prec,0.04_prec/)
!guess(1:7) = (/0.19362_prec,0.12380_prec,0.99998_prec,0.23704_prec,0.36050_prec, 0.00000_prec,-0.00386_prec/)

	
! Simultaneous equations

do i = 2, q
	
	call DNEQNF (solver,tol,1+i*3,5000,guess(1:1+i*3),sol(i,1:1+i*3),fnorm) ! call nles
	
	write(*,*) 'Done with solution',i
	call display(sol(i,:))

	guess = sol(i,1:1+i*3)

end do


End Program tax

