!****************************************************************************************
! One tax
! Optimal taxation
! by Fernando Martin (2003)
!
! Governments sets income tax for today
! Complete Utility Function
! G is endogenous
! LOCAL APPROXIMATION: SOLUTION IS LINEAR, VARIABLES ARE: h,l,t,hk,lk,tk
! Solution is found using simultaneous and nested equations
!****************************************************************************************


Module environment


Implicit none

! Precision parameters

integer,    parameter :: prec     = 8		! double precision
real(prec), parameter :: epsilon  = 1d-13	! auxiliar for IFs
real(prec), parameter :: tol      = 1d-07	! precision of solution
real(prec), parameter :: D        = 5d-06	! maximum increment in all derivatives
integer,    parameter :: ls       = 1		! type of tax (1=income,2=capital,3=labor)


! Global parameters

real(prec), parameter :: beta  = 0.96_prec		! discount factor
real(prec), parameter :: gamma = 0.36_prec		! capital share
real(prec), parameter :: delta = 0.08_prec		! deprecitation rate
real(prec), parameter :: A     = 1.32477_prec 	! TFP parameter

real(prec) :: psi   = 0.000_prec	! elasticity of private-public goods substitution
real(prec) :: rho   = 0.000_prec	! elasticity of consumption-leisure substitution
real(prec) :: sigma = 1.000_prec	! elasticity of intertemporal subsitution

real(prec) :: alpha(3) = (/0.30_prec,0.70_prec,0.13_prec/)	! preference parameters over C, L and G


! NLES variables

real(prec) :: kay, labor, tau
real(prec) :: guess(6), sol(6), solver, fnorm
real(prec) :: nsol, nsol0(2), nsol1(3), nfnorm, nested, nested0, nested1

external :: solver, nested, nested0, nested1, DNEQNF


Contains


!***********************************************************************************
!	FUNCTIONS
!***********************************************************************************


! Production Function

real(prec) function prodfun(k,l)
	Implicit none
	real(prec) :: k, l
	prodfun = A * k**gamma * l**(1-gamma)
end function prodfun


! Wage

real(prec) function wage(k,l)
	Implicit none
	real(prec) :: k, l
	if (k>=0 .and. l>=0) wage = A * (1-gamma) *   k **gamma *   l **(-gamma)
	if (k< 0 .and. l>=0) wage = A * (1-gamma) *-(-k)**gamma *   l **(-gamma)
	if (k>=0 .and. l< 0) wage = A * (1-gamma) *   k **gamma *-(-l)**(-gamma)
	if (k< 0 .and. l< 0) wage = A * (1-gamma) *-(-k)**gamma *-(-l)**(-gamma)
end function wage


! Rate of Return

real(prec) function rate(k,l)
	Implicit none
	real(prec) :: k, l
	if (k>=0 .and. l>=0) rate = A * gamma *   k **(gamma-1) *   l **(1-gamma)
	if (k< 0 .and. l>=0) rate = A * gamma *-(-k)**(gamma-1) *   l **(1-gamma)
	if (k>=0 .and. l< 0) rate = A * gamma *   k **(gamma-1) *-(-l)**(1-gamma)
	if (k< 0 .and. l< 0) rate = A * gamma *-(-k)**(gamma-1) *-(-l)**(1-gamma)
end function rate


! Government Function

real(prec) function govfun(k,l,t)
	Implicit none
	real(prec) :: k, l, t
	if (ls==1) govfun = t * ( (rate(k,l)-delta)*k + wage(k,l)*l )
	if (ls==2) govfun = t *   (rate(k,l)-delta)*k
	if (ls==3) govfun = t * wage(k,l)*l
end function govfun


! Consumption Function

real(prec) function consfun(k,kk,l,t)
	Implicit none
	real(prec) :: k, kk, l, t
	consfun = (rate(k,l)-delta)*k + wage(k,l)*l + k - kk - govfun(k,l,t)
end function consfun


! Marginal Utility for the complete utility function

real(prec) function dU(C,L,G,arg)
	Implicit none
	real(prec) :: C, L, G, XX, YY
	integer    :: arg
	
	if (sigma==1) then
		
		if(arg==1) dU = Uc(C)
		if(arg==2) dU = Ul(L)
		if(arg==3) dU = Ug(G)

	else
	
		YY = alpha(1)*C**rho + alpha(2)*L**rho
		XX = (1-alpha(3)) * YY**(psi/rho) + alpha(3) * G**psi

		if(arg==1) dU = alpha(1)*(1-alpha(3)) * XX**((1-sigma-psi)/psi) * YY**((psi-rho)/rho) * C**(rho-1)
		if(arg==2) dU = alpha(2)*(1-alpha(3)) * XX**((1-sigma-psi)/psi) * YY**((psi-rho)/rho) * L**(rho-1)
		if(arg==3) dU = alpha(3) * XX**((1-sigma-psi)/psi) * G**(psi-1)
	
	end if

end function dU


! Marginal Utility of Consumption

real(prec) function Uc(C)
	Implicit none
	real(prec) :: C
	Uc = alpha(1)*(1-alpha(3)) / C
	if (C<0) Uc=-9999.9_prec
end function Uc


! Marginal Utility of Leisure

real(prec) function Ul(L)
	Implicit none
	real(prec) :: L
	Ul = alpha(2)*(1-alpha(3)) / L
	if (L<0) Ul=-9999.9_prec
end function Ul


! Marginal Utility of Public Good

real(prec) function Ug(G)
	Implicit none
	real(prec) :: G
	Ug = alpha(3) / G
	if (G<0) Ug=-9999.9_prec
end function Ug


!***** Euler Equations and Derivatives *********************************************

! Dynamic Euler Equation

real(prec) function euler(k,kk,kkk,l,ll,t,tt)
	Implicit none
	real(prec) :: k, kk, kkk, l, ll, t, tt, U_c, UU_c
	U_c = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
	UU_c= dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),1)

	if (ls==1) euler = -U_c + beta * UU_c * ((1-tt)*(rate(kk,ll)-delta) + 1)
	if (ls==2) euler = -U_c + beta * UU_c * ((1-tt)*(rate(kk,ll)-delta) + 1)
	if (ls==3) euler = -U_c + beta * UU_c * (rate(kk,ll) - delta + 1)

end function euler


! Static Consumption-Leisure Equation

real(prec) function stateq(k,kk,l,t)
	Implicit none
	real(prec) :: k, kk, l, t, U_c, U_l
	U_c = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
	U_l = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),2)

	if (ls==1) stateq = U_c * (1-t)*wage(k,l) - U_l
	if (ls==2) stateq = U_c *       wage(k,l) - U_l
	if (ls==3) stateq = U_c * (1-t)*wage(k,l) - U_l

end function stateq


! Derivatives of the Euler Equation **********************************

real(prec) function dE(k,kk,kkk,l,ll,t,tt,arg)
	Implicit none
	real(prec) :: k,kk,kkk,l,ll,t,tt
	integer :: arg
	if (arg==1) dE = (euler(k+D,kk,kkk,l,ll,t,tt) - euler(k-D,kk,kkk,l,ll,t,tt)) / (2d-0*D)
	if (arg==2) dE = (euler(k,kk+D,kkk,l,ll,t,tt) - euler(k,kk-D,kkk,l,ll,t,tt)) / (2d-0*D)
	if (arg==3) dE = (euler(k,kk,kkk+D,l,ll,t,tt) - euler(k,kk,kkk-D,l,ll,t,tt)) / (2d-0*D)
	if (arg==4) dE = (euler(k,kk,kkk,l+D,ll,t,tt) - euler(k,kk,kkk,l-D,ll,t,tt)) / (2d-0*D)
	if (arg==5) dE = (euler(k,kk,kkk,l,ll+D,t,tt) - euler(k,kk,kkk,l,ll-D,t,tt)) / (2d-0*D)
	if (arg==6) dE = (euler(k,kk,kkk,l,ll,t+D,tt) - euler(k,kk,kkk,l,ll,t-D,tt)) / (2d-0*D)
	if (arg==7) dE = (euler(k,kk,kkk,l,ll,t,tt+D) - euler(k,kk,kkk,l,ll,t,tt-D)) / (2d-0*D)
end function dE


! Derivatives of the Static Equation *********************************

real(prec) function dS(k,kk,l,t,arg)
	Implicit none
	real(prec) :: k,kk,l,t
	integer :: arg
	if (arg==1) dS = (stateq(k+D,kk,l,t) - stateq(k-D,kk,l,t)) / (2d-0*D)
	if (arg==2) dS = (stateq(k,kk+D,l,t) - stateq(k,kk-D,l,t)) / (2d-0*D)
	if (arg==3) dS = (stateq(k,kk,l+D,t) - stateq(k,kk,l-D,t)) / (2d-0*D)
	if (arg==4) dS = (stateq(k,kk,l,t+D) - stateq(k,kk,l,t-D)) / (2d-0*D)
end function dS


!***** Generalized Euler Equations *************************************************

real(prec) function gee(k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk)

	Implicit none

    ! Define variables

    real(prec) :: k, kk, kkk, kkkk
    real(prec) :: l, ll, lll
    real(prec) :: t, tt, ttt
    real(prec) :: hhk, llk, ttk, ppk
    real(prec) :: U_c, UU_c, U_l, UU_l, U_g, UU_g
    real(prec) :: C_l, C_t, CC_k, CC_l, CC_t
	real(prec) :: G_l, G_t, GG_k, GG_l, GG_t
    real(prec) :: E, X, XX, Y, YY, ZZ
	real(prec) :: a0, a1, a2, a3, a4, a5
    real(prec) :: Eta_kk, Eta_l, Eta_t, Etap_k, Etap_l, Etap_t
    real(prec) :: Eps_kk, Eps_l, Eps_t, Epsp_k, Epsp_l, Epsp_t
    real(prec) :: lamda, lamdap, mu, mup
	

	! Get Utility Function Derivatives

    U_c   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
    U_l   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),2)
	U_g   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),3)

	UU_c  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),1)
    UU_l  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),2)
    UU_g  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),3)


    ! Get Euler Equation Derivatives

    Eta_kk = dE(k,kk,kkk,l,ll,t,tt,2)       +  dE(k,kk,kkk,l,ll,t,tt,3) * hhk + &
         &   dE(k,kk,kkk,l,ll,t,tt,5) * llk +  dE(k,kk,kkk,l,ll,t,tt,7) * ttk
    Eta_l =  dE(k,kk,kkk,l,ll,t,tt,4)
	Eta_t =  dE(k,kk,kkk,l,ll,t,tt,6)
    
    Etap_k = dE(kk,kkk,kkkk,ll,lll,tt,ttt,1)
    Etap_l = dE(kk,kkk,kkkk,ll,lll,tt,ttt,4)
	Etap_t = dE(kk,kkk,kkkk,ll,lll,tt,ttt,6)
    

    ! Get Static Equation Derivatives

    Eps_kk = dS(k,kk,l,t,2)
    Eps_l  = dS(k,kk,l,t,3)
    Eps_t  = dS(k,kk,l,t,4)

    Epsp_k = dS(kk,kkk,ll,tt,1)
    Epsp_l = dS(kk,kkk,ll,tt,3)
    Epsp_t = dS(kk,kkk,ll,tt,4)


    ! Get additional derivatives

	C_l = (consfun(k,kk,l+D,t) - consfun(k,kk,l-D,t)) / (2d-0*D)
	C_t = (consfun(k,kk,l,t+D) - consfun(k,kk,l,t-D)) / (2d-0*D)
	
	CC_k = (consfun(kk+D,kkk,ll,tt) - consfun(kk-D,kkk,ll,tt)) / (2d-0*D)
	CC_l = (consfun(kk,kkk,ll+D,tt) - consfun(kk,kkk,ll-D,tt)) / (2d-0*D)
	CC_t = (consfun(kk,kkk,ll,tt+D) - consfun(kk,kkk,ll,tt-D)) / (2d-0*D)

	G_l = (govfun(k,l+D,t) - govfun(k,l-D,t)) / (2d-0*D)
	G_t = (govfun(k,l,t+D) - govfun(k,l,t-D)) / (2d-0*D)
	
	GG_k = (govfun(kk+D,ll,tt) - govfun(kk-D,ll,tt)) / (2d-0*D)
	GG_l = (govfun(kk,ll+D,tt) - govfun(kk,ll-D,tt)) / (2d-0*D)
	GG_t = (govfun(kk,ll,tt+D) - govfun(kk,ll,tt-D)) / (2d-0*D)


	! Exact solutions

	if (ls==1) then

		C_l  = (1-t) * wage(k,l)
		CC_l = (1-tt) * wage(kk,ll)
		C_t  = -((rate(k,l)-delta) * k + wage(k,l) * l )
		CC_t = -((rate(kk,ll)-delta) * kk + wage(kk,ll) * ll)
		CC_k = (1-tt) * (rate(kk,ll)-delta) + 1

		G_l  = t * wage(k,l)
		GG_l = tt * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (rate(kk,ll)-delta)

	end if

	if (ls==2) then

		C_l  = (1-t*gamma) * wage(k,l)
		CC_l = (1-tt*gamma) * wage(kk,ll)
		C_t  = -(rate(k,l)-delta) * k
		CC_t = -(rate(kk,ll)-delta) * kk
		CC_k = (1-tt) * (gamma*rate(kk,ll)-delta) + (1-gamma)*rate(kk,ll) + 1

		G_l  = t * gamma * wage(k,l)
		GG_l = tt * gamma * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (gamma*rate(kk,ll)-delta)

	end if

	if (ls==3) then

		C_l  = (1-t*(1-gamma)) * wage(k,l)
		CC_l = (1-tt*(1-gamma)) * wage(kk,ll)
		C_t  = -wage(k,l) * l
		CC_t = -wage(kk,ll) * ll
		CC_k = (1-tt*(1-gamma)) * rate(kk,ll) + 1 - delta

		G_l  = t * (1-gamma) * wage(k,l)
		GG_l = tt * (1-gamma) * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (1-gamma) * rate(kk,ll)

	end if

	
	! Set coefficients of GEE

    a0 = (Eps_kk*Eta_l - Eps_l*Eta_kk) / (Eps_l*Eta_t - Eps_t*Eta_l)
    a1 = (Eps_t*Eta_kk - Eps_kk*Eta_t) / (Eps_l*Eta_t - Eps_t*Eta_l)
    a2 = beta
    a3 = beta * (Epsp_k*Etap_l - Epsp_l*Etap_k) / (Epsp_l*Etap_t - Epsp_t*Etap_l)
    a4 = beta * (Epsp_t*Etap_k - Epsp_k*Etap_t) / (Epsp_l*Etap_t - Epsp_t*Etap_l)


    ! Define expressions

	E = -U_c + beta * UU_c * CC_k
    X  = U_c  * C_t  + U_g  * G_t
    XX = UU_c * CC_t + UU_g * GG_t
    Y  = U_c  * C_l  - U_l  + U_g  * G_l
    YY = UU_c * CC_l - UU_l + UU_g * GG_l
    ZZ = UU_g * GG_k
	

	! Get GEE
	
	GEE = E + a0 * X + a1 * Y + a2 * ZZ + a3 * XX + a4 * YY

end function gee


! Derivatives of GEE *********************************

real(prec) function dGEE(k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk,arg)
	Implicit none
	real(prec) :: k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,p,pp,ppp,hhk,llk,ttk
	integer :: arg
	if (arg== 1) dGEE = (GEE(k+D,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k-D,kk,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 2) dGEE = (GEE(k,kk+D,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk-D,kkk,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 3) dGEE = (GEE(k,kk,kkk+D,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk-D,kkkk,l,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 4) dGEE = (GEE(k,kk,kkk,kkkk+D,l,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk-D,l,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 5) dGEE = (GEE(k,kk,kkk,kkkk,l+D,ll,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l-D,ll,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 6) dGEE = (GEE(k,kk,kkk,kkkk,l,ll+D,lll,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll-D,lll,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 7) dGEE = (GEE(k,kk,kkk,kkkk,l,ll,lll+D,t,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll,lll-D,t,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 8) dGEE = (GEE(k,kk,kkk,kkkk,l,ll,lll,t+D,tt,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll,lll,t-D,tt,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg== 9) dGEE = (GEE(k,kk,kkk,kkkk,l,ll,lll,t,tt+D,ttt,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll,lll,t,tt-D,ttt,hhk,llk,ttk)) / (2d-0*D)
	if (arg==10) dGEE = (GEE(k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt+D,hhk,llk,ttk) - GEE(k,kk,kkk,kkkk,l,ll,lll,t,tt,ttt-D,hhk,llk,ttk)) / (2d-0*D)
end function dGEE


End Module environment



!*********************************************************************************************



!******************************************************************************************
!	SOLVER SUBROUTINE
!******************************************************************************************


subroutine solver(X,F,n)

	use environment
	Implicit none

	integer :: n
	real(prec) :: x(n), F(n)

	real(prec) :: k,l,t,hk,lk,tk

	k  = x(1)
	l  = x(2)
	t  = x(3)
	hk = x(4)
	lk = x(5)
	tk = x(6)


	! System of Equations

	F(1) = euler(k,k,k,l,l,t,t)
	
	F(2) = stateq(k,k,l,t)
	
	F(3) = gee(k,k,k,k,l,l,l,t,t,t,hk,lk,tk)

	F(4) =  dE(k,k,k,l,l,t,t,1) + dE(k,k,k,l,l,t,t,2)*hk + dE(k,k,k,l,l,t,t,3)*hk*hk &
		        			    + dE(k,k,k,l,l,t,t,4)*lk + dE(k,k,k,l,l,t,t,5)*lk*hk &
							    + dE(k,k,k,l,l,t,t,6)*tk + dE(k,k,k,l,l,t,t,7)*tk*hk

	F(5) =  dS(k,k,l,t,1) + dS(k,k,l,t,2)*hk + dS(k,k,l,t,3)*lk + dS(k,k,l,t,4)*tk

	F(6) = dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 1)          &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 2)*hk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 3)*hk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 4)*hk*hk*hk &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 5)*lk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 6)*lk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 7)*lk*hk*hk &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 8)*tk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 9)*tk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk,10)*tk*hk*hk
	
end subroutine solver



!******************************************************************************************
!	NESTED SUBROUTINE
!******************************************************************************************


subroutine nested(X,F,n)

	use environment
	Implicit none

	integer :: n
	real(prec) :: x(n), F(n)

	real(prec) :: k,l,t,hk,lk,tk

	k   = x(1)
	kay = k

	call DNEQNF (nested0, tol, 2, 5000, guess(2:3), nsol0, nfnorm) ! call nles

	l = nsol0(1)
	t = nsol0(2)

	labor = l
	tau   = t

	call DNEQNF (nested1, tol, 3, 5000, guess(4:6), nsol1, nfnorm) ! call nles

	hk = nsol1(1)
	lk = nsol1(2)
	tk = nsol1(3)

	guess(2:3) = nsol0
	guess(4:6) = nsol1

	
	! System of Equations

	F(1) = dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 1)          &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 2)*hk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 3)*hk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 4)*hk*hk*hk &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 5)*lk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 6)*lk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 7)*lk*hk*hk &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 8)*tk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 9)*tk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk,10)*tk*hk*hk
	
end subroutine nested


subroutine nested0(X,F,n)

	use environment
	Implicit none

	integer :: n
	real(prec) :: x(n), F(n)

	real(prec) :: k,l,t,hk,lk,tk

	k = kay
	l = x(1)
	t = x(2)

	F(1) = euler(k,k,k,l,l,t,t)
	
	F(2) = stateq(k,k,l,t)
	
end subroutine nested0


subroutine nested1(X,F,n)

	use environment
	Implicit none

	integer :: n
	real(prec) :: x(n), F(n)

	real(prec) :: k,l,t,hk,lk,tk

	k = kay
	l = labor
	t = tau

	hk = x(1)
	lk = x(2)
	tk = x(3)

	F(1) = gee(k,k,k,k,l,l,l,t,t,t,hk,lk,tk)

	F(2) =  dE(k,k,k,l,l,t,t,1) + dE(k,k,k,l,l,t,t,2)*hk + dE(k,k,k,l,l,t,t,3)*hk*hk &
		        			    + dE(k,k,k,l,l,t,t,4)*lk + dE(k,k,k,l,l,t,t,5)*lk*hk &
							    + dE(k,k,k,l,l,t,t,6)*tk + dE(k,k,k,l,l,t,t,7)*tk*hk

	F(3) =  dS(k,k,l,t,1) + dS(k,k,l,t,2)*hk + dS(k,k,l,t,3)*lk + dS(k,k,l,t,4)*tk
	
end subroutine nested1



!******************************************************************************************
!	DISPLAY SUBROUTINE
!******************************************************************************************


subroutine display(x)

	use environment
	Implicit none

	real(prec) :: x(6), F(6)

	real(prec) :: k,l,t,hk,lk,tk

	k  = x(1)
	l  = x(2)
	t  = x(3)
	hk = x(4)
	lk = x(5)
	tk = x(6)


	! System of Equations

	F(1) = euler(k,k,k,l,l,t,t)
	
	F(2) = stateq(k,k,l,t)
	
	F(3) = gee(k,k,k,k,l,l,l,t,t,t,hk,lk,tk)

	F(4) =  dE(k,k,k,l,l,t,t,1) + dE(k,k,k,l,l,t,t,2)*hk + dE(k,k,k,l,l,t,t,3)*hk*hk &
		        			    + dE(k,k,k,l,l,t,t,4)*lk + dE(k,k,k,l,l,t,t,5)*lk*hk &
							    + dE(k,k,k,l,l,t,t,6)*tk + dE(k,k,k,l,l,t,t,7)*tk*hk

	F(5) =  dS(k,k,l,t,1) + dS(k,k,l,t,2)*hk + dS(k,k,l,t,3)*lk + dS(k,k,l,t,4)*tk

	F(6) = dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 1)          &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 2)*hk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 3)*hk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 4)*hk*hk*hk &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 5)*lk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 6)*lk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 7)*lk*hk*hk &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 8)*tk       &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk, 9)*tk*hk    &
		 + dGEE(k,k,k,k,l,l,l,t,t,t,hk,lk,tk,10)*tk*hk*hk
	
	write(5,"(F20.16)") x(1:3)
	write(5,*)
	write(5,"(F20.16)") x(4:6)
	write(5,*)
	write(5,"(F20.16)") F
	write(5,*)

end subroutine display



!******************************************************************************************
!	PROGRAM
!******************************************************************************************


Program tax

use environment
use imslf90

call ERSET (0, 1, 0)	! tells ISLM not to stop if there's a fatal error

open(unit=5,file='tax.out')			! solutions
open(unit=3,file='solution.out')	! interpolated solutions


! Make guess

if (ls==1) guess = (/1.836_prec,0.258_prec,0.254_prec,0.850_prec,-0.047_prec,0.041_prec/)
if (ls==2) guess = (/0.551_prec,0.285_prec,0.821_prec,0.799_prec,-0.067_prec,0.623_prec/)
if (ls==3) guess = (/2.127_prec,0.252_prec,0.297_prec,0.844_prec,-0.041_prec,0.019_prec/)
	
	
! Simultaneous equations

call DNEQNF (solver, tol, 6, 5000, guess, sol, fnorm) ! call nles

call display(sol)

!guess = sol


! Nested equations

call DNEQNF (nested, tol, 1, 5000, guess(1), nsol, fnorm) ! call nles
kay = nsol
call DNEQNF (nested0, tol, 2, 5000, guess(2:3), nsol0, nfnorm) ! call nles
labor = nsol0(1)
tau   = nsol0(2)
call DNEQNF (nested1, tol, 3, 5000, guess(4:6), nsol1, nfnorm) ! call nles

sol = (/kay,labor,tau,nsol1(1),nsol1(2),nsol1(3)/)

call display(sol)


End Program tax

