!****************************************************************************************
! One tax
! Optimal taxation
! by Fernando Martin (2003)
!
! Governments sets income tax for today
! Complete Utility Function
! G is endogenous
! GLOBAL APPROXIMATION
! Simultaneous equations: guess on (h,l,t)
!****************************************************************************************


Module environment


Implicit none

! Precision parameters

integer,    parameter :: prec     = 8		! double precision
real(prec), parameter :: epsilon  = 1d-13	! auxiliar for IFs
real(prec), parameter :: tol      = 1d-09	! precision of solution
real(prec), parameter :: globtol  = 1d-09	! precision of global approximation
real(prec), parameter :: D        = 5d-06	! maximum increment in all derivatives
integer,    parameter :: spltype  = 1		! type of spline (1=int, 2=akm, etc)
integer,    parameter :: gridtype = 2		! type of grid (1 linear, 2 exponential)
integer,    parameter :: ls       = 1		! type of tax (1=income,2=capital,3=labor)


! Set global parameters

real(prec), parameter :: beta  = 0.96_prec		! discount factor
real(prec), parameter :: gamma = 0.36_prec		! capital share
real(prec), parameter :: delta = 0.09_prec		! deprecitation rate
real(prec), parameter :: A     = 1.32477_prec 	! TFP parameter

real(prec) :: psi   = 0.000_prec	! elasticity of private-public goods substitution
real(prec) :: rho   = 0.000_prec	! elasticity of consumption-leisure substitution
real(prec) :: sigma = 1.000_prec	! elasticity of intertemporal subsitution

real(prec), dimension(1:3) :: alpha = (/0.30_prec,0.7_prec,0.13_prec/)	! preference parameters over C, L and G


! Set Grid variables

integer, parameter :: m = 100		! size of the b-grid
integer :: gridpoint				! element in the b-grid
real(prec), dimension(m) :: grid	! state variable grid
real(prec) :: kmin = 0.10_prec		! minimum K
real(prec) :: kmax = 3.00_prec		! maximum K

! Other variables

integer :: j, iter
  
real(prec) :: int, r1=1.0_prec, r2=1.0_prec, r3=1.0_prec, r4=1.0_prec

real(prec), dimension(m) :: FH, FL, FT, rH, rL, rT, rX, guess, diffS, sS

real(prec), dimension(4,m) :: cH, cL, cT, cX, sH, sL, sT
real(prec), dimension(m) :: diffH, diffL, diffT
integer, parameter :: deg = 2, sdeg = 3		! degree of polynomial of fitted curve
real(prec) :: co(3,deg+1), sspoly(deg+1), stat(3,10), stata(12)

real(prec), dimension(m) :: guessgrid, Hguess, Lguess, Tguess

real(prec), dimension(1000) :: grid_k		! output in 'solution.out'


! Non-linear solver settings

real(prec) :: kay, labor, theta, tau	! guess and equations

integer :: nnsolver, nout, nout2
real(prec) :: solver, solver0, solver1, steady_state, ssfvec

real(prec), dimension(3) :: sol , fvec		! nles output variables
real(prec), dimension(3) :: sol1, fvec1

external :: solver, solver1, steady_state, DNEQNF, DNEQBF, umach, DRLINE


! Interpolation settings

real(prec) :: DSCINT, DSCAKM, DCSDER, DSPLEZ
external   :: DSCINT, DSCAKM, DCSDER, DSPLEZ

! Guess to solutions

real(prec), dimension(3,m) :: xguess
real(prec), dimension(3) :: xguess1 = (/ 0.787_prec,-0.119_prec,-1.246_prec /)


!real(prec), dimension(3) :: xguess  = (/ 2.2853_prec, 0.2644_prec, 0.4220_prec /)	! solutions for Gbar=0, gy=0.2
!real(prec), dimension(3) :: xguess1 = (/ 0.852_prec,-0.028_prec, 0.163_prec /)

Contains


!***********************************************************************************
!	FUNCTIONS
!***********************************************************************************


! Production Function

real(prec) function prodfun(k,l)
	Implicit none
	real(prec) :: k, l
	prodfun = A * k**gamma * l**(1-gamma)
end function prodfun


! Wage

real(prec) function wage(k,l)
	Implicit none
	real(prec) :: k, l
	if (k>=0 .and. l>=0) wage = A * (1-gamma) *   k **gamma *   l **(-gamma)
	if (k< 0 .and. l>=0) wage = A * (1-gamma) *-(-k)**gamma *   l **(-gamma)
	if (k>=0 .and. l< 0) wage = A * (1-gamma) *   k **gamma *-(-l)**(-gamma)
	if (k< 0 .and. l< 0) wage = A * (1-gamma) *-(-k)**gamma *-(-l)**(-gamma)
end function wage


! Rate of Return

real(prec) function rate(k,l)
	Implicit none
	real(prec) :: k, l
	if (k>=0 .and. l>=0) rate = A * gamma *   k **(gamma-1) *   l **(1-gamma)
	if (k< 0 .and. l>=0) rate = A * gamma *-(-k)**(gamma-1) *   l **(1-gamma)
	if (k>=0 .and. l< 0) rate = A * gamma *   k **(gamma-1) *-(-l)**(1-gamma)
	if (k< 0 .and. l< 0) rate = A * gamma *-(-k)**(gamma-1) *-(-l)**(1-gamma)
end function rate


! Government Function

real(prec) function govfun(k,l,t)
	Implicit none
	real(prec) :: k, l, t
	if (ls==1) govfun = t * ( (rate(k,l)-delta)*k + wage(k,l)*l )
	if (ls==2) govfun = t *   (rate(k,l)-delta)*k
	if (ls==3) govfun = t * wage(k,l)*l
end function govfun


! Consumption Function

real(prec) function consfun(k,kk,l,t)
	Implicit none
	real(prec) :: k, kk, l, t
	consfun = (rate(k,l)-delta)*k + wage(k,l)*l + k - kk - govfun(k,l,t)
end function consfun


! Marginal Utility for the complete utility function

real(prec) function dU(C,L,G,arg)
	Implicit none
	real(prec) :: C, L, G, XX, YY
	integer    :: arg
	
	if (sigma==1) then
		
		if(arg==1) dU = Uc(C)
		if(arg==2) dU = Ul(L)
		if(arg==3) dU = Ug(G)

	else
	
		YY = alpha(1)*C**rho + alpha(2)*L**rho
		XX = (1-alpha(3)) * YY**(psi/rho) + alpha(3) * G**psi

		if(arg==1) dU = alpha(1)*(1-alpha(3)) * XX**((1-sigma-psi)/psi) * YY**((psi-rho)/rho) * C**(rho-1)
		if(arg==2) dU = alpha(2)*(1-alpha(3)) * XX**((1-sigma-psi)/psi) * YY**((psi-rho)/rho) * L**(rho-1)
		if(arg==3) dU = alpha(3) * XX**((1-sigma-psi)/psi) * G**(psi-1)
	
	end if

end function dU


! Marginal Utility of Consumption

real(prec) function Uc(C)
	Implicit none
	real(prec) :: C
	Uc = alpha(1)*(1-alpha(3)) / C
	if (C<0) Uc=-9999.9_prec
end function Uc


! Marginal Utility of Leisure

real(prec) function Ul(L)
	Implicit none
	real(prec) :: L
	Ul = alpha(2)*(1-alpha(3)) / L
	if (L<0) Ul=-9999.9_prec
end function Ul


! Marginal Utility of Public Good

real(prec) function Ug(G)
	Implicit none
	real(prec) :: G
	Ug = alpha(3) / G
	if (G<0) Ug=-9999.9_prec
end function Ug


!***** Euler Equations and Derivatives *********************************************

! Dynamic Euler Equation

real(prec) function euler(k,kk,kkk,l,ll,t,tt)
	Implicit none
	real(prec) :: k, kk, kkk, l, ll, t, tt, U_c, UU_c
	U_c = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
	UU_c= dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),1)

	if (ls==1) euler = -U_c + beta * UU_c * ((1-tt)*(rate(kk,ll)-delta) + 1)
	if (ls==2) euler = -U_c + beta * UU_c * ((1-tt)*(rate(kk,ll)-delta) + 1)
	if (ls==3) euler = -U_c + beta * UU_c * (rate(kk,ll) - delta + 1)

end function euler


! Static Consumption-Leisure Equation

real(prec) function stateq(k,kk,l,t)
	Implicit none
	real(prec) :: k, kk, l, t, U_c, U_l
	U_c = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
	U_l = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),2)

	if (ls==1) stateq = U_c * (1-t)*wage(k,l) - U_l
	if (ls==2) stateq = U_c *       wage(k,l) - U_l
	if (ls==3) stateq = U_c * (1-t)*wage(k,l) - U_l

end function stateq


! Derivatives of the Euler Equation **********************************

real(prec) function dE(k,kk,kkk,l,ll,t,tt,arg)
	Implicit none
	real(prec) :: k,kk,kkk,l,ll,t,tt
	integer :: arg
	if (arg==1) dE = (euler(k+D,kk,kkk,l,ll,t,tt) - euler(k-D,kk,kkk,l,ll,t,tt)) / (2d-0*D)
	if (arg==2) dE = (euler(k,kk+D,kkk,l,ll,t,tt) - euler(k,kk-D,kkk,l,ll,t,tt)) / (2d-0*D)
	if (arg==3) dE = (euler(k,kk,kkk+D,l,ll,t,tt) - euler(k,kk,kkk-D,l,ll,t,tt)) / (2d-0*D)
	if (arg==4) dE = (euler(k,kk,kkk,l+D,ll,t,tt) - euler(k,kk,kkk,l-D,ll,t,tt)) / (2d-0*D)
	if (arg==5) dE = (euler(k,kk,kkk,l,ll+D,t,tt) - euler(k,kk,kkk,l,ll-D,t,tt)) / (2d-0*D)
	if (arg==6) dE = (euler(k,kk,kkk,l,ll,t+D,tt) - euler(k,kk,kkk,l,ll,t-D,tt)) / (2d-0*D)
	if (arg==7) dE = (euler(k,kk,kkk,l,ll,t,tt+D) - euler(k,kk,kkk,l,ll,t,tt-D)) / (2d-0*D)
end function dE


! Derivatives of the Static Equation *********************************

real(prec) function dS(k,kk,l,t,arg)
	Implicit none
	real(prec) :: k,kk,l,t
	integer :: arg
	if (arg==1) dS = (stateq(k+D,kk,l,t) - stateq(k-D,kk,l,t)) / (2d-0*D)
	if (arg==2) dS = (stateq(k,kk+D,l,t) - stateq(k,kk-D,l,t)) / (2d-0*D)
	if (arg==3) dS = (stateq(k,kk,l+D,t) - stateq(k,kk,l-D,t)) / (2d-0*D)
	if (arg==4) dS = (stateq(k,kk,l,t+D) - stateq(k,kk,l,t-D)) / (2d-0*D)
end function dS


!***** Generalized Euler Equations *************************************************

real(prec) function gee(k,kk,kkk,l,ll,t,tt,hhk,llk,ttk)

	Implicit none

    ! Define variables

    real(prec) :: k, kk, kkk
    real(prec) :: l, ll
    real(prec) :: t, tt
    real(prec) :: hhk, llk, ttk, ppk
    real(prec) :: U_c, UU_c, U_l, UU_l, U_g, UU_g, UU_cc
    real(prec) :: C_l, C_t, CC_k, CC_l, CC_t
	real(prec) :: G_l, G_t, GG_k, GG_l, GG_t
    real(prec) :: E, X, XX, Y, YY, ZZ
	real(prec) :: a0, a1, a2, a3, a4, a5
    real(prec) :: Eta_kk, Eta_l, Eta_t, Etap_k, Etap_l, Etap_t
    real(prec) :: Eps_kk, Eps_l, Eps_t, Epsp_k, Epsp_l, Epsp_t
    real(prec) :: lamda, lamdap, mu, mup
	

	! Get Utility Function Derivatives

    U_c   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),1)
    U_l   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),2)
	U_g   = dU(consfun(k,kk,l,t),1-l,govfun(k,l,t),3)

	UU_c  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),1)
    UU_l  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),2)
    UU_g  = dU(consfun(kk,kkk,ll,tt),1-ll,govfun(kk,ll,tt),3)

	UU_cc =-alpha(1)*(1-alpha(3)) / (consfun(kk,kkk,ll,tt)**2)


    ! Get Euler Equation Derivatives

    Eta_kk = dE(k,kk,kkk,l,ll,t,tt,2)       +  dE(k,kk,kkk,l,ll,t,tt,3) * hhk + &
         &   dE(k,kk,kkk,l,ll,t,tt,5) * llk +  dE(k,kk,kkk,l,ll,t,tt,7) * ttk
    Eta_l =  dE(k,kk,kkk,l,ll,t,tt,4)
	Eta_t =  dE(k,kk,kkk,l,ll,t,tt,6)
    
    !Etap_k = dE(kk,kkk,kkkk,ll,lll,tt,ttt,1)
    !Etap_l = dE(kk,kkk,kkkk,ll,lll,tt,ttt,4)
	!Etap_t = dE(kk,kkk,kkkk,ll,lll,tt,ttt,6)
    

    ! Get Static Equation Derivatives

    Eps_kk = dS(k,kk,l,t,2)
    Eps_l  = dS(k,kk,l,t,3)
    Eps_t  = dS(k,kk,l,t,4)

    Epsp_k = dS(kk,kkk,ll,tt,1)
    Epsp_l = dS(kk,kkk,ll,tt,3)
    Epsp_t = dS(kk,kkk,ll,tt,4)


    ! Get additional derivatives

	C_l = (consfun(k,kk,l+D,t) - consfun(k,kk,l-D,t)) / (2d-0*D)
	C_t = (consfun(k,kk,l,t+D) - consfun(k,kk,l,t-D)) / (2d-0*D)
	
	CC_k = (consfun(kk+D,kkk,ll,tt) - consfun(kk-D,kkk,ll,tt)) / (2d-0*D)
	CC_l = (consfun(kk,kkk,ll+D,tt) - consfun(kk,kkk,ll-D,tt)) / (2d-0*D)
	CC_t = (consfun(kk,kkk,ll,tt+D) - consfun(kk,kkk,ll,tt-D)) / (2d-0*D)

	G_l = (govfun(k,l+D,t) - govfun(k,l-D,t)) / (2d-0*D)
	G_t = (govfun(k,l,t+D) - govfun(k,l,t-D)) / (2d-0*D)
	
	GG_k = (govfun(kk+D,ll,tt) - govfun(kk-D,ll,tt)) / (2d-0*D)
	GG_l = (govfun(kk,ll+D,tt) - govfun(kk,ll-D,tt)) / (2d-0*D)
	GG_t = (govfun(kk,ll,tt+D) - govfun(kk,ll,tt-D)) / (2d-0*D)


	! Exact solutions

	if (ls==1) then

		C_l  = (1-t) * wage(k,l)
		CC_l = (1-tt) * wage(kk,ll)
		C_t  = -((rate(k,l)-delta) * k + wage(k,l) * l )
		CC_t = -((rate(kk,ll)-delta) * kk + wage(kk,ll) * ll)
		CC_k = (1-tt) * (rate(kk,ll)-delta) + 1

		G_l  = t * wage(k,l)
		GG_l = tt * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (rate(kk,ll)-delta)

	end if

	if (ls==2) then

		C_l  = (1-t*gamma) * wage(k,l)
		CC_l = (1-tt*gamma) * wage(kk,ll)
		C_t  = -(rate(k,l)-delta) * k
		CC_t = -(rate(kk,ll)-delta) * kk
		CC_k = (1-tt) * (gamma*rate(kk,ll)-delta) + (1-gamma)*rate(kk,ll) + 1

		G_l  = t * gamma * wage(k,l)
		GG_l = tt * gamma * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (gamma*rate(kk,ll)-delta)

	end if

	if (ls==3) then

		C_l  = (1-t*(1-gamma)) * wage(k,l)
		CC_l = (1-tt*(1-gamma)) * wage(kk,ll)
		C_t  = -wage(k,l) * l
		CC_t = -wage(kk,ll) * ll
		CC_k = (1-tt*(1-gamma)) * rate(kk,ll) + 1 - delta

		G_l  = t * (1-gamma) * wage(k,l)
		GG_l = tt * (1-gamma) * wage(kk,ll)
		G_t  = -C_t
		GG_t = -CC_t
		GG_k = tt * (1-gamma) * rate(kk,ll)

	end if


	! More exact solutions

	Etap_k = -UU_cc * CC_k
	Etap_l = -UU_cc * CC_l
	Etap_t = -UU_cc * CC_t

	
	! Set coefficients of GEE

    a0 = (Eps_kk*Eta_l - Eps_l*Eta_kk) / (Eps_l*Eta_t - Eps_t*Eta_l)
    a1 = (Eps_t*Eta_kk - Eps_kk*Eta_t) / (Eps_l*Eta_t - Eps_t*Eta_l)
    a2 = beta
    a3 = beta * (Epsp_k*Etap_l - Epsp_l*Etap_k) / (Epsp_l*Etap_t - Epsp_t*Etap_l)
    a4 = beta * (Epsp_t*Etap_k - Epsp_k*Etap_t) / (Epsp_l*Etap_t - Epsp_t*Etap_l)


    ! Define expressions

	E = -U_c + beta * UU_c * CC_k
    X  = U_c  * C_t  + U_g  * G_t
    XX = UU_c * CC_t + UU_g * GG_t
    Y  = U_c  * C_l  - U_l  + U_g  * G_l
    YY = UU_c * CC_l - UU_l + UU_g * GG_l
    ZZ = UU_g * GG_k
	

	! Get GEE
	
	GEE = E + a0 * X + a1 * Y + a2 * ZZ + a3 * XX + a4 * YY

end function gee


! Derivatives of GEE *********************************

real(prec) function dGEE(k,kk,kkk,l,ll,t,tt,hhk,llk,ttk,arg)
	Implicit none
	real(prec) :: k,kk,kkk,l,ll,t,tt,p,pp,ppp,hhk,llk,ttk
	integer :: arg
	if (arg==1) dGEE = (GEE(k+D,kk,kkk,l,ll,t,tt,hhk,llk,ttk) - GEE(k-D,kk,kkk,l,ll,t,tt,hhk,llk,ttk)) / (2d-0*D)
	if (arg==2) dGEE = (GEE(k,kk+D,kkk,l,ll,t,tt,hhk,llk,ttk) - GEE(k,kk-D,kkk,l,ll,t,tt,hhk,llk,ttk)) / (2d-0*D)
	if (arg==3) dGEE = (GEE(k,kk,kkk+D,l,ll,t,tt,hhk,llk,ttk) - GEE(k,kk,kkk-D,l,ll,t,tt,hhk,llk,ttk)) / (2d-0*D)
	if (arg==4) dGEE = (GEE(k,kk,kkk,l+D,ll,t,tt,hhk,llk,ttk) - GEE(k,kk,kkk,l-D,ll,t,tt,hhk,llk,ttk)) / (2d-0*D)
	if (arg==5) dGEE = (GEE(k,kk,kkk,l,ll+D,t,tt,hhk,llk,ttk) - GEE(k,kk,kkk,l,ll-D,t,tt,hhk,llk,ttk)) / (2d-0*D)
	if (arg==6) dGEE = (GEE(k,kk,kkk,l,ll,t+D,tt,hhk,llk,ttk) - GEE(k,kk,kkk,l,ll,t-D,tt,hhk,llk,ttk)) / (2d-0*D)
	if (arg==7) dGEE = (GEE(k,kk,kkk,l,ll,t,tt+D,hhk,llk,ttk) - GEE(k,kk,kkk,l,ll,t,tt-D,hhk,llk,ttk)) / (2d-0*D)
end function dGEE



!***** Spline evaluation ***********************************************************

real(prec) function seval(x,R,C,arg)
	
	Implicit none
	integer :: arg
	real(prec) :: x, DCSDER
	real(prec), dimension(m) :: R
	real(prec), dimension(4,m) :: C

	seval = DCSDER(arg,x,m-1,R,C)
 
end function seval



!***** SPLINE SUBROUTINE ***********************************************************

subroutine spline(X)

	Implicit none

	real(prec), dimension(m) :: X

	if (spltype==1) call DCSINT (m, grid, X, rX, cX)
	if (spltype==2) call DCSAKM (m, grid, X, rX, cX)

end subroutine spline



End Module environment



!*********************************************************************************************



!******************************************************************************************
!	SOLVER SUBROUTINE
!******************************************************************************************


subroutine solver(X,F,nn)
!subroutine solver(nn,X,F)


use environment
Implicit none


!***** Setup ***********************************************************************

integer :: nn		! number of variables
real(prec), dimension(nn) :: X,F	! guess and equations

real(prec) :: k, h, hh, l, ll, t, tt
real(prec) :: hhk,llk,ttk


!***** Main Loop *******************************************************************

nnsolver = nnsolver + 1

k = grid(gridpoint)
h = x(1)
l = x(2)
t = x(3)

hh  = seval(h ,rH,cH,0)
ll  = seval(h ,rL,cL,0)
tt  = seval(h ,rT,cT,0)

hhk = seval(h ,rH,cH,1)
llk = seval(h ,rL,cL,1)
ttk = seval(h ,rT,cT,1)


! System of Equations

f(1) = euler(k,h,hh,l,ll,t,tt)

f(2) = stateq(k,h,l,t)

f(3) = gee(k,h,hh,l,ll,t,tt,hhk,llk,ttk)

	
!write(5,"(7F10.5)") f(1), f(2), f(3), k, h, l, t


end subroutine solver



!******************************************************************************************
!	SOLVER SUBROUTINE
!******************************************************************************************


subroutine solver1(X,F,nn)


use environment
Implicit none


!***** Setup ***********************************************************************

integer :: nn		! number of variables
real(prec), dimension(nn) :: X,F	! guess and equations

real(prec) :: k, l, t
real(prec) :: hk,lk,tk


!***** Main Loop *******************************************************************

nnsolver = nnsolver + 1

k = x(1)
l = x(2)
t = x(3)

hk = seval(k,rH,cH,1)
lk = seval(k,rL,cL,1)
tk = seval(k,rT,cT,1)


! System of Equations

f(1) = euler(k,k,k,l,l,t,t)

f(2) = stateq(k,k,l,t)

f(3) = gee(k,k,k,l,l,t,t,hk,lk,tk)


!write(5,"(9F10.5)") f(1), f(2), f(3), k, l, t, hk, lk, tk
	

end subroutine solver1



!******************************************************************************************
!	STEADY STATE SUBROUTINE
!******************************************************************************************


subroutine steady_state(X,F,nn)

use environment
Implicit none


!***** Setup ***********************************************************************

integer :: nn		! number of variables
real(prec) :: X,F	! guess and equations

F = x - seval(x,rH,cH,0)

end subroutine steady_state



!******************************************************************************************
!	VERIFY SUBROUTINE
!******************************************************************************************


subroutine verify(k)

Use environment
Implicit none

real(prec) :: k,h,hh,l,ll,t,tt,hhk,llk,ttk
real(prec) :: k1,h1,hh1,l1,ll1,t1,tt1,hhk1,llk1,ttk1
real(prec) :: k2,h2,hh2,l2,ll2,t2,tt2,hhk2,llk2,ttk2

h   = seval(k ,rH,cH,0)
l   = seval(k ,rL,cL,0)
t   = seval(k ,rT,cT,0)
hh  = seval(h ,rH,cH,0)
ll  = seval(h ,rL,cL,0)
tt  = seval(h ,rT,cT,0)

hhk = seval(h ,rH,cH,1)
llk = seval(h ,rL,cL,1)
ttk = seval(h ,rT,cT,1)

k1   = k - D
h1   = seval(k1 ,rH,cH,0)
l1   = seval(k1 ,rL,cL,0)
t1   = seval(k1 ,rT,cT,0)
hh1  = seval(h1 ,rH,cH,0)
ll1  = seval(h1 ,rL,cL,0)
tt1  = seval(h1 ,rT,cT,0)

hhk1 = seval(h1 ,rH,cH,1)
llk1 = seval(h1 ,rL,cL,1)
ttk1 = seval(h1 ,rT,cT,1)

k2   = k + D
h2   = seval(k2 ,rH,cH,0)
l2   = seval(k2 ,rL,cL,0)
t2   = seval(k2 ,rT,cT,0)
hh2  = seval(h2 ,rH,cH,0)
ll2  = seval(h2 ,rL,cL,0)
tt2  = seval(h2 ,rT,cT,0)

hhk2 = seval(h2 ,rH,cH,1)
llk2 = seval(h2 ,rL,cL,1)
ttk2 = seval(h2 ,rT,cT,1)



write(5,*)
write(5,*) euler(k,h,hh,l,ll,t,tt)

write(5,*) stateq(k,h,l,t)

write(5,*) gee(k,h,hh,l,ll,t,tt,hhk,llk,ttk)

write(5,*) (euler(k2,h2,hh2,l2,ll2,t2,tt2) - euler(k1,h1,hh1,l1,ll1,t1,tt1)) / (2d-0*D)

write(5,*) (stateq(k2,h2,l2,t2) - stateq(k1,h1,l1,t1)) / (2d-0*D)

write(5,*) (gee(k2,h2,hh2,l2,ll2,t2,tt2,hhk2,llk2,ttk2) - gee(k1,h1,hh1,l1,ll1,t1,tt1,hhk1,llk1,ttk1)) / (2d-0*D)


end subroutine verify



!******************************************************************************************
!	GEE SERIES SUBROUTINE
!******************************************************************************************


subroutine gee_series

Use environment
Implicit none

real(prec) :: k,h,hh,l,ll,t,tt,hhk,llk,ttk
real(prec) :: FG(m), rG(m), cG(4,m)
integer :: i

do i = 1, m

	k   = grid(i)	
	h   = seval(k ,rH,cH,0)
	l   = seval(k ,rL,cL,0)
	t   = seval(k ,rT,cT,0)
	hh  = seval(h ,rH,cH,0)
	ll  = seval(h ,rL,cL,0)
	tt  = seval(h ,rT,cT,0)
	
	hhk = seval(h ,rH,cH,1)
	llk = seval(h ,rL,cL,1)
	ttk = seval(h ,rT,cT,1)

	FG(i) = gee(k,h,hh,l,ll,t,tt,hhk,llk,ttk)

end do

call spline(FG)
rG = rX
cG = cX

k = kmin
int = (kmax-kmin) / 999

do i = 1, 1000
	
	write(4,"(5F20.16)") k,seval(k,rG,cG,0),seval(k,rG,cG,1),seval(k,rG,cG,2),seval(k,rG,cG,3)
	k = k + int

end do

end subroutine gee_series



!******************************************************************************************
!	PROGRAM
!******************************************************************************************


Program tax

use environment
use imslf90

call ERSET (0, 1, 0)	! tells ISLM not to stop if there's a fatal error

open(unit=5,file='tax.out')			! solutions
open(unit=3,file='solution.out')	! interpolated solutions
open(unit=4,file='gee.out')	! interpolated solutions


! Define grid of K

grid(1) = kmin
grid(m) = kmax

if (gridtype == 1) then
  
	int = (grid(m)-grid(1)) / (m-1)
	do j = 2, m-1
		grid(j) = grid(j-1) + int
	end do

end if

if (gridtype == 2) then
  
	int = (grid(m)/grid(1))**(1d-0/(m-1)) - 1
	do j = 2, m-1
		grid(j) = grid(j-1) * (1+int)
	end do

end if


! Guess solutions for h(k), l(k) and theta(k) using solution in steady state

if (ls==1) then

	do j = 1,m
		xguess(1,j) = 0.21355_prec + 0.89891_prec * grid(j) - 0.00833_prec * grid(j)**2
		xguess(2,j) = 0.37979_prec - 0.08292_prec * grid(j) + 0.00913_prec * grid(j)**2
		xguess(3,j) = 0.15143_prec + 0.07232_prec * grid(j) - 0.00877_prec * grid(j)**2
	end do

end if


if (ls==2) then

	do j = 1,m
		xguess(1,j) = 0.14149_prec + 0.80451_prec * grid(j)
		xguess(2,j) = 0.32548_prec - 0.04468_prec * grid(j)
		xguess(3,j) = 0.45032_prec + 0.49560_prec * grid(j)
	end do

end if


if (ls==3) then

	do j = 1,m
		xguess(1,j) = 0.20094_prec + 0.97229_prec * grid(j) - 0.02710_prec * grid(j)**2
		xguess(2,j) = 0.39784_prec - 0.09581_prec * grid(j) + 0.01184_prec * grid(j)**2
		xguess(3,j) = 0.22613_prec + 0.04797_prec * grid(j) - 0.00632_prec * grid(j)**2
	end do

end if


call spline(xguess(1,:))
rH = rX
cH = cX

call spline(xguess(2,:))
rL = rX
cL = cX

call spline(xguess(3,:))
rT = rX
cT = cX

!write(5,*) xguess
!write(5,*)


! Find steady state with guess

!call DNEQNF (solver1, tol, 3, 5000, (/2.64631_prec,0.25709_prec,0.25612_prec/), sol, fvec)
!write(5,*)
!write(5,*) sol
!write(5,*)


!***** Global Approximation *************************************************************


do while (r4>globtol .and. iter<500)
  
	iter = iter + 1

	write(5,"(A,I4)") 'This is iteration ',iter
	write(*,"(A,I4)") 'This is iteration ',iter

	
	do j = 1, m

		gridpoint = j
		!write(5,*)
		!write(5,"(4F10.5)") grid(j),xguess(:,j)

		call DNEQNF (solver, tol, 3, 5000, xguess(:,j), sol, fvec) ! call nles
		FH(j) = sol(1)
		FL(j) = sol(2)
		FT(j) = sol(3)
		
		!write(3,"(7F15.10)") grid(j),xguess(:,j),FH(j),FL(j),FT(j)
		xguess(:,j) = sol	! use solution as next iteration guess

	end do
		!write(3,*)

	! Compute cubic spline interpolant for h(k), l(k) and theta(k)
		
	call spline(FH)
	rH = rX
	cH = cX

	call spline(FL)
	rL = rX
	cL = cX

	call spline(FT)
	rT = rX
	cT = cX


	! Evaluate interpolation

	write(5,*)
	write(5,"(11x,A,7x,A,5x,A,5x,A)") 'k','h(k)','l(k)','t(k)'

    do j = 1,m

		diffH(j) = abs(cH(1,j) - sH(1,j))
		diffL(j) = abs(cL(1,j) - sL(1,j))
		diffT(j) = abs(cT(1,j) - sT(1,j))
						
		write(5,"(I5,4F9.3)") j,grid(j),FH(j),FL(j),FT(j)
				
    end do

	
	! Check convergence

    r1 = maxval(diffH,DIM=1)
	r2 = maxval(diffL,DIM=1)
	r3 = maxval(diffT,DIM=1)
	r4 = max(r1,r2,r3)
	    
    write(*,"(A,4F15.10)") 'Diff:',r1,r2,r3,r4
    write(5,"(A,4F15.10)") 'Diff:',r1,r2,r3,r4
    write(*,*)
	write(5,*)'_____________________________________________________________________________'
    write(5,*)
    write(5,*)

	
	sH = cH  ! store new functions
	sL = cL  ! store new functions
	sT = cT  ! store new functions
	sS = FH


end do


write(5,*) 'Global Approximation done'
write(*,*) 'Global Approximation done'


!***** Global Approximation Done ********************************************************


! Get grid of solutions by interpolation

grid_k(1)    = grid(1)
grid_k(1000) = grid(m)
int = (grid(m)-grid(1))/999

do j = 1, 1000
	
	if (j>1) grid_k(j) = grid_k(j-1)+int
	write(3,"(5F15.10)") grid_k(j),seval(grid_k(j),rH,cH,0),seval(grid_k(j),rL,cL,0),seval(grid_k(j),rT,cT,0),govfun(grid_k(j),seval(grid_k(j),rL,cL,0),seval(grid_k(j),rT,cT,0))

end do


! Find steady state

call DNEQNF (steady_state, tol, 1, 5000, 2.66_prec, kay, ssfvec) ! call nles

labor = seval(kay,rL,cL,0)
tau   = seval(kay,rT,cT,0)

write(5,"(4F10.5)") kay,seval(kay,rH,cH,0),labor,tau
write(5,"(3F10.5)") seval(kay,rH,cH,1),seval(kay,rL,cL,1),seval(kay,rT,cT,1)
write(3,*)
write(3,"(4F15.10)") kay,seval(kay,rH,cH,0),labor,tau
write(3,"(3F15.10)") seval(kay,rH,cH,1),seval(kay,rL,cL,1),seval(kay,rT,cT,1)
write(3,"(3F15.10)") euler(kay,kay,kay,labor,labor,tau,tau), stateq(kay,kay,labor,tau), &
& gee(kay,kay,kay,labor,labor,tau,tau,seval(kay,rH,cH,1),seval(kay,rL,cL,1),seval(kay,rT,cT,1))

write(5,*)
write(5,*) kay
write(5,*) labor
write(5,*) tau
write(5,*)
write(5,*) seval(kay,rH,cH,1)
write(5,*) seval(kay,rL,cL,1)
write(5,*) seval(kay,rT,cT,1)
write(5,*)
write(5,*) rate(kay,labor)
write(5,*) wage(kay,labor)
write(5,*) prodfun(kay,labor)
write(5,*) consfun(kay,kay,labor,tau)
write(5,*) govfun(kay,labor,tau)
write(5,*)
write(5,*) euler(kay,kay,kay,labor,labor,tau,tau)
write(5,*) stateq(kay,kay,labor,tau)
write(5,*) gee(kay,kay,kay,labor,labor,tau,tau,seval(kay,rH,cH,1),seval(kay,rL,cL,1),seval(kay,rT,cT,1))


! Check Equations

call verify(kay)


! Get GEE and derivatives

call gee_series


! Get Fit of Functions

call DRCURV (m,grid,FH,deg,co(1,:),sspoly,stat(1,:))
call DRCURV (m,grid,FL,deg,co(2,:),sspoly,stat(2,:))
call DRCURV (m,grid,Ft,deg,co(3,:),sspoly,stat(3,:))


write(5,*)
write(5,*) 'Fit of Functions'
write(5,"(4F10.5)") stat(1,5),stat(2,5),stat(3,5)
write(5,*)

write(5,"(A)") 'Var		Coefficients'
write(5,"(A,3F10.5)") 'K',co(1,:)
write(5,"(A,3F10.5)") 'L',co(2,:)
write(5,"(A,3F10.5)") 'T',co(3,:)


End Program tax

